@@ -1,13 +1,16 @@
 
 from django.contrib.auth.models import User, Group
 from django.test import TestCase
+from django.test.client import Client
 
 from object_permissions import *
-from object_permissions.registration import TestModel, UnknownPermissionException
-
+from object_permissions.registration import TestModel, TestModelChild, \
+    TestModelChildChild, UnknownPermissionException
+from object_permissions.views.permissions import ObjectPermissionForm, \
+    ObjectPermissionFormNewUsers
 
 class TestModelPermissions(TestCase):
-    perms = [u'Perm1', u'Perm2', u'Perm3', u'Perm4']
+    perms = set(['Perm1', 'Perm2', 'Perm3', 'Perm4'])
 
     def setUp(self):
         self.tearDown()
@@ -35,12 +38,32 @@ class TestModelPermissions(TestCase):
 
     def tearDown(self):
         TestModel.objects.all().delete()
+        TestModelChild.objects.all().delete()
+        TestModelChildChild.objects.all().delete()
         User.objects.all().delete()
         Group.objects.all().delete()
 
     def test_trivial(self):
         pass
 
+    def test_registration(self):
+        """
+        Test that permissions were registered correctly
+        """
+        perms1 = get_model_perms(TestModel)
+        perms2 = get_model_perms(TestModelChild)
+        
+        self.assert_('Perm1' in perms1)
+        self.assert_('Perm2' in perms1)
+        self.assert_('Perm3' in perms1)
+        self.assert_('Perm4' in perms1)
+        
+        self.assert_(isinstance(perms2, (dict,)))
+        self.assert_('Perm1' in perms2)
+        self.assert_('Perm2' in perms2)
+        self.assert_('Perm3' in perms2)
+        self.assert_('Perm4' in perms2)
+
     def test_grant_user_permissions(self):
         """
         Grant a user permissions
@@ -125,45 +148,45 @@ class TestModelPermissions(TestCase):
         
         # revoke single perm
         revoke(user0, 'Perm1', object0)
-        self.assertEqual([u'Perm2', u'Perm3', u'Perm4'], get_user_perms(user0, object0))
-        self.assertEqual(perms, get_user_perms(user0, object1))
-        self.assertEqual(perms, get_user_perms(user1, object0))
-        self.assertEqual(perms, get_user_perms(user1, object1))
+        self.assertEqual(set(['Perm2', u'Perm3', 'Perm4']), set(get_user_perms(user0, object0)))
+        self.assertEqual(perms, set(get_user_perms(user0, object1)))
+        self.assertEqual(perms, set(get_user_perms(user1, object0)))
+        self.assertEqual(perms, set(get_user_perms(user1, object1)))
         
         # revoke a second perm
         revoke(user0, 'Perm3', object0)
-        self.assertEqual([u'Perm2', u'Perm4'], get_user_perms(user0, object0))
-        self.assertEqual(perms, get_user_perms(user0, object1))
-        self.assertEqual(perms, get_user_perms(user1, object0))
-        self.assertEqual(perms, get_user_perms(user1, object1))
+        self.assertEqual(set(['Perm2', 'Perm4']), set(get_user_perms(user0, object0)))
+        self.assertEqual(perms, set(get_user_perms(user0, object1)))
+        self.assertEqual(perms, set(get_user_perms(user1, object0)))
+        self.assertEqual(perms, set(get_user_perms(user1, object1)))
         
         # revoke from another object
         revoke(user0, 'Perm3', object1)
-        self.assertEqual([u'Perm2', u'Perm4'], get_user_perms(user0, object0))
-        self.assertEqual([u'Perm1', u'Perm2', u'Perm4'], get_user_perms(user0, object1))
-        self.assertEqual(perms, get_user_perms(user1, object0))
-        self.assertEqual(perms, get_user_perms(user1, object1))
+        self.assertEqual(set(['Perm2', 'Perm4']), set(get_user_perms(user0, object0)))
+        self.assertEqual(set(['Perm1', 'Perm2', 'Perm4']), set(get_user_perms(user0, object1)))
+        self.assertEqual(perms, set(get_user_perms(user1, object0)))
+        self.assertEqual(perms, set(get_user_perms(user1, object1)))
         
         # revoke from another user
         revoke(user1, 'Perm4', object0)
-        self.assertEqual([u'Perm2', u'Perm4'], get_user_perms(user0, object0))
-        self.assertEqual([u'Perm1', u'Perm2', u'Perm4'], get_user_perms(user0, object1))
-        self.assertEqual([u'Perm1', u'Perm2', u'Perm3'], get_user_perms(user1, object0))
-        self.assertEqual(perms, get_user_perms(user1, object1))
+        self.assertEqual(set(['Perm2', 'Perm4']), set(get_user_perms(user0, object0)))
+        self.assertEqual(set(['Perm1', 'Perm2', 'Perm4']), set(get_user_perms(user0, object1)))
+        self.assertEqual(set(['Perm1', 'Perm2', u'Perm3']), set(get_user_perms(user1, object0)))
+        self.assertEqual(perms, set(get_user_perms(user1, object1)))
         
         # revoke perm user does not have
         revoke(user0, 'Perm1', object0)
-        self.assertEqual([u'Perm2', u'Perm4'], get_user_perms(user0, object0))
-        self.assertEqual([u'Perm1', u'Perm2', u'Perm4'], get_user_perms(user0, object1))
-        self.assertEqual([u'Perm1', u'Perm2', u'Perm3'], get_user_perms(user1, object0))
-        self.assertEqual(perms, get_user_perms(user1, object1))
+        self.assertEqual(set(['Perm2', 'Perm4']), set(get_user_perms(user0, object0)))
+        self.assertEqual(set(['Perm1', 'Perm2', 'Perm4']), set(get_user_perms(user0, object1)))
+        self.assertEqual(set(['Perm1', 'Perm2', u'Perm3']), set(get_user_perms(user1, object0)))
+        self.assertEqual(perms, set(get_user_perms(user1, object1)))
         
         # revoke perm that does not exist
         revoke(user0, 'DoesNotExist', object0)
-        self.assertEqual([u'Perm2', u'Perm4'], get_user_perms(user0, object0))
-        self.assertEqual([u'Perm1', u'Perm2', u'Perm4'], get_user_perms(user0, object1))
-        self.assertEqual([u'Perm1', u'Perm2', u'Perm3'], get_user_perms(user1, object0))
-        self.assertEqual(perms, get_user_perms(user1, object1))
+        self.assertEqual(set(['Perm2', 'Perm4']), set(get_user_perms(user0, object0)))
+        self.assertEqual(set(['Perm1', 'Perm2', 'Perm4']), set(get_user_perms(user0, object1)))
+        self.assertEqual(set(['Perm1', 'Perm2', u'Perm3']), set(get_user_perms(user1, object0)))
+        self.assertEqual(perms, set(get_user_perms(user1, object1)))
     
     def test_revoke_all(self):
         """
@@ -182,21 +205,21 @@ class TestModelPermissions(TestCase):
         
         revoke_all(user0, object0)
         self.assertEqual([], get_user_perms(user0, object0))
-        self.assertEqual(perms, get_user_perms(user0, object1))
-        self.assertEqual(perms, get_user_perms(user1, object0))
-        self.assertEqual(perms, get_user_perms(user1, object1))
+        self.assertEqual(perms, set(get_user_perms(user0, object1)))
+        self.assertEqual(perms, set(get_user_perms(user1, object0)))
+        self.assertEqual(perms, set(get_user_perms(user1, object1)))
         
         revoke_all(user0, object1)
         self.assertEqual([], get_user_perms(user0, object0))
         self.assertEqual([], get_user_perms(user0, object1))
-        self.assertEqual(perms, get_user_perms(user1, object0))
-        self.assertEqual(perms, get_user_perms(user1, object1))
+        self.assertEqual(perms, set(get_user_perms(user1, object0)))
+        self.assertEqual(perms, set(get_user_perms(user1, object1)))
         
         revoke_all(user1, object0)
         self.assertEqual([], get_user_perms(user0, object0))
         self.assertEqual([], get_user_perms(user0, object1))
         self.assertEqual([], get_user_perms(user1, object0))
-        self.assertEqual(perms, get_user_perms(user1, object1))
+        self.assertEqual(perms, set(get_user_perms(user1, object1)))
         
         revoke_all(user1, object1)
         self.assertEqual([], get_user_perms(user0, object0))
@@ -214,23 +237,23 @@ class TestModelPermissions(TestCase):
         object1 = self.object1
         
         perms1 = self.perms
-        perms2 = ['Perm1', 'Perm2']
-        perms3 = ['Perm2', 'Perm3']
+        perms2 = set(['Perm1', 'Perm2'])
+        perms3 = set(['Perm2', 'Perm3'])
         perms4 = []
 
         # grant single property
         set_user_perms(user0, perms1, object0)
-        self.assertEqual(perms1, get_user_perms(user0, object0))
+        self.assertEqual(perms1, set(get_user_perms(user0, object0)))
         self.assertEqual([], get_user_perms(user0, object1))
         self.assertEqual([], get_user_perms(user1, object0))
         
         set_user_perms(user0, perms2, object0)
-        self.assertEqual(perms2, get_user_perms(user0, object0))
+        self.assertEqual(perms2, set(get_user_perms(user0, object0)))
         self.assertEqual([], get_user_perms(user0, object1))
         self.assertEqual([], get_user_perms(user1, object0))
         
         set_user_perms(user0, perms3, object0)
-        self.assertEqual(perms3, get_user_perms(user0, object0))
+        self.assertEqual(perms3, set(get_user_perms(user0, object0)))
         self.assertEqual([], get_user_perms(user0, object1))
         self.assertEqual([], get_user_perms(user1, object0))
         
@@ -243,13 +266,13 @@ class TestModelPermissions(TestCase):
         
         set_user_perms(user0, perms2, object1)
         self.assertEqual(perms4, get_user_perms(user0, object0))
-        self.assertEqual(perms2, get_user_perms(user0, object1))
+        self.assertEqual(perms2, set(get_user_perms(user0, object1)))
         self.assertEqual([], get_user_perms(user1, object0))
         
         set_user_perms(user1, perms1, object0)
         self.assertEqual(perms4, get_user_perms(user0, object0))
-        self.assertEqual(perms2, get_user_perms(user0, object1))
-        self.assertEqual(perms1, get_user_perms(user1, object0))
+        self.assertEqual(perms2, set(get_user_perms(user0, object1)))
+        self.assertEqual(perms1, set(get_user_perms(user1, object0)))
     
     def test_has_perm(self):
         """
@@ -459,37 +482,37 @@ class TestModelPermissions(TestCase):
         
         # grant single property
         grant(user0, 'Perm1', object0)
-        self.assertEqual([u'Perm1'], get_user_perms(user0, object0))
+        self.assertEqual(['Perm1'], get_user_perms(user0, object0))
         self.assertEqual([], get_user_perms(user0, object1))
         self.assertEqual([], get_user_perms(user1, object0))
         self.assertEqual([], get_user_perms(user1, object1))
         
         # grant property again
         grant(user0, 'Perm1', object0)
-        self.assertEqual([u'Perm1'], get_user_perms(user0, object0))
+        self.assertEqual(['Perm1'], get_user_perms(user0, object0))
         self.assertEqual([], get_user_perms(user0, object1))
         self.assertEqual([], get_user_perms(user1, object0))
         self.assertEqual([], get_user_perms(user1, object1))
         
         # grant second property
         grant(user0, 'Perm2', object0)
-        self.assertEqual([u'Perm1', u'Perm2'], get_user_perms(user0, object0))
+        self.assertEqual(set(['Perm1', 'Perm2']), set(get_user_perms(user0, object0)))
         self.assertEqual([], get_user_perms(user0, object1))
         self.assertEqual([], get_user_perms(user1, object0))
         self.assertEqual([], get_user_perms(user1, object1))
         
         # grant property to another object
         grant(user0, 'Perm2', object1)
-        self.assertEqual([u'Perm1', u'Perm2'], get_user_perms(user0, object0))
-        self.assertEqual([u'Perm2'], get_user_perms(user0, object1))
+        self.assertEqual(set(['Perm1', 'Perm2']), set(get_user_perms(user0, object0)))
+        self.assertEqual(['Perm2'], get_user_perms(user0, object1))
         self.assertEqual([], get_user_perms(user1, object0))
         self.assertEqual([], get_user_perms(user1, object1))
         
         # grant perms to other user
         grant(user1, 'Perm3', object0)
-        self.assertEqual([u'Perm1', u'Perm2'], get_user_perms(user0, object0))
-        self.assertEqual([u'Perm2'], get_user_perms(user0, object1))
-        self.assertEqual([u'Perm3'], get_user_perms(user1, object0))
+        self.assertEqual(set(['Perm1', 'Perm2']), set(get_user_perms(user0, object0)))
+        self.assertEqual(['Perm2'], get_user_perms(user0, object1))
+        self.assertEqual(['Perm3'], get_user_perms(user1, object0))
         self.assertEqual([], get_user_perms(user1, object1))
     
     def test_get_objects_any_perms(self):
@@ -549,6 +572,68 @@ class TestModelPermissions(TestCase):
         self.assert_(object1 in query)
         self.assertEqual(2, query.count())
     
+    def test_get_objects_any_perms_related(self):
+        """
+        Test retrieving objects with any matching perms and related model
+        options
+        """
+        object2 = TestModel.objects.create(name='test2')
+        object2.save()
+        
+        child0 = TestModelChild.objects.create(parent=object0)
+        child1 = TestModelChild.objects.create(parent=object1)
+        child2 = TestModelChild.objects.create(parent=object2)
+        child3 = TestModelChild.objects.create(parent=object2)
+        child0.save()
+        child1.save()
+        child2.save()
+        
+        childchild = TestModelChildChild.objects.create(parent=child0)
+        childchild.save()
+        
+        user0.grant('Perm1', object0)  # perms on both
+        user0.grant('Perm2', child0)   # perms on both
+        user0.grant('Perm3', object1)  # perm on parent only (child 1)
+        user0.grant('Perm4', child2)   # perm on child only
+        user0.grant('Perm1', childchild)
+        
+        # related field with implicit perms
+        query = user0.get_objects_any_perms(TestModelChild, parent=None)
+        self.assertEqual(3, len(query))
+        self.assert_(child0 in query, 'user should have perms on parent and directly')
+        self.assert_(child1 in query, 'user should have perms on parent')
+        self.assert_(child2 in query, 'user should have perms on parent, and directly')
+        self.assertFalse(child3 in query, 'user should have no perms on this object or its parent')
+        
+        # related field with single perms
+        query = user0.get_objects_any_perms(TestModelChild, parent=['Perm3'])
+        self.assertEqual(3, len(query))
+        self.assert_(child0 in query, 'user should have perms on parent and directly')
+        self.assert_(child1 in query, 'user should have perms on parent')
+        self.assert_(child2 in query, 'user should have perms on parent')
+        self.assertFalse(child3 in query, 'user should have no perms on this object or its parent')
+        
+        # related field with multiple perms
+        query = user0.get_objects_any_perms(TestModelChild, parent=['Perm1','Perm3'])
+        self.assertEqual(3, len(query))
+        self.assert_(child0 in query, 'user should have perms on parent and directly')
+        self.assert_(child1 in query, 'user should have perms on parent')
+        self.assert_(child2 in query, 'user should have perms on parent')
+        self.assertFalse(child3 in query, 'user should have no perms on this object or its parent')
+        
+        # mix of direct and related perms
+        query = user0.get_objects_any_perms(TestModelChild, perms=['Perm4'], parent=['Perm1'])
+        self.assertEqual(2, len(query))
+        self.assert_(child0 in query, 'user should have perms on parent and directly')
+        self.assertFalse(child1 in query, 'user should not have perms on parent')
+        self.assert_(child2 in query, 'user should have perms directly')
+        self.assertFalse(child3 in query, 'user should have no perms on this object or its parent')
+        
+        # multiple relations
+        query = user0.get_objects_any_perms(TestModelChildChild, parent=['Perm2'], parent__parent=['Perm1'])
+        self.assertEqual(1, len(query))
+        self.assert_(childchild in query)
+    
     def test_get_objects_all_perms(self):
         """
         Test retrieving objects that have all matching perms
@@ -599,6 +684,57 @@ class TestModelPermissions(TestCase):
         self.assertFalse(object1 in query)
         self.assertEqual(1, query.count())
     
+    def test_get_objects_all_perms_related(self):
+        """
+        Test retrieving objects with all matching perms and related model
+        options
+        """
+        child0 = TestModelChild.objects.create(parent=object0)
+        child1 = TestModelChild.objects.create(parent=object0)
+        child2 = TestModelChild.objects.create(parent=object1)
+        child0.save()
+        child1.save()
+        child2.save()
+        
+        childchild = TestModelChildChild.objects.create(parent=child0)
+        childchild.save()
+        
+        user0.grant('Perm1', object0)
+        user0.grant('Perm1', object1)
+        user0.grant('Perm2', object1)
+        
+        user0.grant('Perm1', child0)
+        user0.grant('Perm1', child1)
+        user0.grant('Perm2', child1)
+        user0.grant('Perm1', childchild)
+        
+        # related field with single perms
+        query = user0.get_objects_all_perms(TestModelChild, perms=['Perm1'], parent=['Perm1'])
+        self.assertEqual(2, len(query))
+        self.assert_(child0 in query)
+        self.assert_(child1 in query)
+        self.assertFalse(child2 in query)
+        
+        # related field with single perms - has parent but not child
+        query = user0.get_objects_all_perms(TestModelChild, perms=['Perm4'], parent=['Perm1'])
+        self.assertEqual(0, len(query))
+        
+        # related field with single perms - has child but not parent
+        query = user0.get_objects_all_perms(TestModelChild, perms=['Perm1'], parent=['Perm4'])
+        self.assertEqual(0, len(query))
+        
+        # related field with multiple perms
+        query = user0.get_objects_all_perms(TestModelChild, perms=['Perm1'], parent=['Perm1','Perm2'])
+        self.assertEqual(1, len(query))
+        self.assertFalse(child0 in query)
+        self.assert_(child1 in query)
+        self.assertFalse(child2 in query)
+        
+        # multiple relations
+        query = user0.get_objects_all_perms(TestModelChildChild, perms=['Perm1'], parent=['Perm1'], parent__parent=['Perm1'])
+        self.assertEqual(1, len(query))
+        self.assert_(childchild in query)
+    
     def test_get_all_objects_any_perms(self):
         """
         Test retrieving all objects from all models
@@ -861,4 +997,208 @@ class TestModelPermissions(TestCase):
         self.assertFalse(user_has_all_perms(user0, object0, ['Perm3'], groups=False))
         
         # perm on group, checking groups
-        self.assert_(user_has_all_perms(user0, object0, ['Perm3']))
\ No newline at end of file
+        self.assert_(user_has_all_perms(user0, object0, ['Perm3']))
+
+
+class TestPermissionViews(TestCase):
+    """ tests for user specific test views """
+    
+    def setUp(self):
+        self.tearDown()
+        
+        user0 = User(id=2, username='tester0')
+        user0.set_password('secret')
+        user0.save()
+        user1 = User(id=3, username='tester1')
+        user1.set_password('secret')
+        user1.save()
+        
+        d = globals()
+        d['c'] = Client()
+        d['user0'] = user0
+        d['user1'] = user1
+    
+    def tearDown(self):
+        User.objects.all().delete()
+    
+    def test_permissions_all(self):
+        """ tests view for returning all permissions across all objects """
+        url = '/user/%s/permissions/all'
+        
+        # anonymous user
+        response = c.get(url % user1.pk, follow=True)
+        self.assertEqual(200, response.status_code)
+        self.assertTemplateUsed(response, 'registration/login.html')
+        
+        # unauthorized user
+        self.assert_(c.login(username=user0.username, password='secret'))
+        response = c.get(url % user1.pk)
+        self.assertEqual(403, response.status_code)
+        
+        # unknown user
+        user0.is_superuser = True
+        user0.save()
+        response = c.get(url % 123456)
+        self.assertEqual(404, response.status_code)
+        
+        # superuser
+        response = c.get(url % user1.pk)
+        self.assertEqual(200, response.status_code)
+        self.assertTemplateUsed(response, 'object_permissions/permissions/objects.html')
+
+
+class TestObjectPermissionForm(TestCase):
+    """ Tests for testing forms for editing permissions """
+    
+    def setUp(self):
+        self.tearDown()
+        
+        obj = TestModel.objects.create()
+        child = TestModelChild.objects.create()
+        user = User.objects.create(username='tester')
+        group = Group.objects.create(name='test_group')
+        
+        d = globals()
+        d['obj'] = obj
+        d['child'] = child
+        d['user'] = user
+        d['group'] = group
+    
+    def tearDown(self):
+        if 'obj' in globals():
+            if 'user' in globals():
+                user.revoke_all(obj)
+            if 'group' in globals():
+                group.revoke_all(obj)
+        TestModel.objects.all().delete()
+        TestModelChild.objects.all().delete()
+        User.objects.all().delete()
+        Group.objects.all().delete()
+    
+    def test_trivial(self):
+        form = ObjectPermissionForm(obj)
+    
+    def test_choices_generation(self):
+        """ tests that permissions choices lists are generated correctly """
+        choices = ObjectPermissionForm.get_choices(obj)
+        
+        self.assertEqual(4, len(choices))
+        choice3, choice2, choice1, choice4 = choices
+        
+        perm, display = choice1
+        self.assertEqual('Perm1', perm)
+        self.assertEqual(display, {'label':'Perm One','description':'The first permission'})
+        
+        perm, display = choice2
+        self.assertEqual('Perm2', perm)
+        self.assertEqual(display, {'label':'Perm2','description':'The second permission'})
+        
+        perm, display = choice3
+        self.assertEqual('Perm3', perm)
+        self.assertEqual(display, {'label':'Perm Three'})
+        
+        perm, display = choice4
+        self.assertEqual('Perm4', perm)
+        self.assertEqual(display, {'label':'Perm4'})
+    
+    def test_choices_cache(self):
+        """ tests that choices lists are cached """
+        choices = ObjectPermissionForm.get_choices(obj)
+        choices2 = ObjectPermissionForm.get_choices(obj)
+        choices3 = ObjectPermissionForm.get_choices(obj)
+        choices4 = ObjectPermissionForm.get_choices(obj)
+        
+        self.assertEqual(id(choices), id(choices2))
+        self.assertEqual(id(choices3), id(choices4))
+    
+    def test_invalid_grantee(self):
+        """ tests entering bad id for group or user """
+        data = {'user':1234, 'permissions':['Perm1']}
+        form = ObjectPermissionForm(obj, data)
+        self.assertFalse(form.is_valid())
+        
+        data = {'group':1234, 'permissions':['Perm1']}
+        form = ObjectPermissionForm(obj, data)
+        self.assertFalse(form.is_valid())
+    
+    def test_user_group_exclusivity(self):
+        """ tests that only a user or a group can be selected """
+        
+        data = {'user':user.pk, 'group':group.pk, 'permissions':['Perm1']}
+        form = ObjectPermissionForm(obj, data)
+        self.assertFalse(form.is_valid())
+        
+        data = {'user':user.pk, 'permissions':['Perm1']}
+        form = ObjectPermissionForm(obj, data)
+        self.assert_(form.is_valid(), form.errors)
+        self.assertEqual(user, form.cleaned_data['grantee'])
+        
+        data = {'group':group.pk, 'permissions':['Perm1']}
+        form = ObjectPermissionForm(obj, data)
+        self.assert_(form.is_valid())
+        self.assertEqual(group, form.cleaned_data['grantee'])
+
+
+class TestObjectPermissionFormNewUsers(TestCase):
+    
+    def setUp(self):
+        self.tearDown()
+        
+        obj = TestModel.objects.create()
+        user = User.objects.create(username='tester')
+        
+        d = globals()
+        d['obj'] = obj
+        d['user'] = user
+    
+    def tearDown(self):
+        if 'user' in globals():
+            user.revoke_all(obj)
+        TestModel.objects.all().delete()
+        TestModelChild.objects.all().delete()
+        User.objects.all().delete()
+    
+    def test_trivial(self):
+        form = ObjectPermissionFormNewUsers(obj)
+
+
+    def test_new_user(self):
+        """
+        Tests adding a new user
+        
+        validates:
+            * perms must be included
+        """
+        data = {'user':user.pk}
+        form = ObjectPermissionFormNewUsers(obj, data)
+        self.assertFalse(form.is_valid())
+        
+        data = {'user':user.pk, 'permissions':[]}
+        form = ObjectPermissionFormNewUsers(obj, data)
+        self.assertFalse(form.is_valid())
+
+        data = {'user':user.pk, 'permissions':['Perm1']}
+        form = ObjectPermissionFormNewUsers(obj, data)
+        self.assert_(form.is_valid())
+        self.assertTrue(form.cleaned_data['new'])
+    
+    def test_modify_user(self):
+        """
+        Tests modifying a user's perms
+        """
+        user.grant('Perm1', obj)
+        data = {'user':user.pk, 'permissions':['Perm1']}
+        form = ObjectPermissionFormNewUsers(obj, data)
+        self.assert_(form.is_valid())
+        self.assertFalse(form.cleaned_data['new'])
+        
+        data = {'user':user.pk}
+        form = ObjectPermissionFormNewUsers(obj, data)
+        self.assert_(form.is_valid())
+        self.assertFalse(form.cleaned_data['new'])
+        
+        user.grant('Perm1', obj)
+        data = {'user':user.pk, 'permissions':[]}
+        form = ObjectPermissionFormNewUsers(obj, data)
+        self.assert_(form.is_valid())
+        self.assertFalse(form.cleaned_data['new'])
\ No newline at end of file
